-------------------------------------------------------------------------------
-- TDC Core / CERN
-------------------------------------------------------------------------------
--
-- unit name: tdc_package
--
-- author: Sebastien Bourdeauducq, sebastien@milkymist.org
--
-- description: Component declarations for the TDC core
--
-- references: http://www.ohwr.org/projects/tdc-core
--
-------------------------------------------------------------------------------
-- last changes:
-- 2011-11-07 SB Pre-inversion
-- 2011-11-05 SB Added extra histogram bits support
-- 2011-10-25 SB Added single/multi channel bank components
-- 2011-08-03 SB Created file
-------------------------------------------------------------------------------

-- Copyright (C) 2011 CERN
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, version 3 of the License.
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

-- DESCRIPTION:
-- This contains component declarations for all the modules of the TDC core.
-- It is used both internally to instantiate modules, and by the user to
-- instantiate the top-level "tdc" module.

library ieee;
use ieee.std_logic_1164.all;

package tdc_package is

  type t_tdc_timestamp is record
    slope : std_logic;
    channel : std_logic_vector(2 downto 0);
    frac : std_logic_vector(11 downto 0);
    coarse : std_logic_vector(31 downto 0);
    tai : std_logic_vector(31 downto 0);
    seq : std_logic_vector(31 downto 0);
  end record;

  type t_tdc_timestamp_array is array(integer range<>) of t_tdc_timestamp;

  component CARRY4 is
    port (
      CO     : out std_logic_vector(3 downto 0);
      O      : out std_logic_vector(3 downto 0);
      CI     : in  std_ulogic := 'L';
      CYINIT : in  std_ulogic := 'L';
      DI     : in  std_logic_vector(3 downto 0);
      S      : in  std_logic_vector(3 downto 0));
  end component CARRY4;

  component FDR is
    generic (
      INIT : bit);
    port (
      Q : out std_ulogic;
      C : in  std_ulogic;
      D : in  std_ulogic;
      R : in  std_ulogic);
  end component FDR;

  component LUT2 is
    generic (
      INIT : bit_vector);
    port (
      O  : out std_ulogic;
      I0 : in  std_ulogic;
      I1 : in  std_ulogic);
  end component LUT2;

  component LUT1 is
    generic (
      INIT : bit_vector);
    port (
      O  : out std_ulogic;
      I0 : in  std_ulogic);
  end component LUT1;
  
end package;
