#include "board.h"
#include "gpio.h"

#define GPIO_REG_COR 0
#define GPIO_REG_SOR 4
#define GPIO_REG_DDR 8
#define GPIO_REG_PSR 12

#define GPIO_BANK_SIZE 32

static void wb_gpio_set_dir(const struct gpio_pin *pin, int dir)
{
  void *base = (void*)pin->rwops->priv;
  volatile uint32_t *dr = base + (GPIO_BANK_SIZE * (pin->pin >> 5)) + GPIO_REG_DDR;
  uint32_t mask = 1 << (pin->pin & 0x1f);

  if (dir)
    *dr |= mask;
  else
    *dr &= ~mask;
}

static void wb_gpio_out(const struct gpio_pin *pin, int value)
{
  void *base = (void*)pin->rwops->priv;
  volatile void *regs = base + (GPIO_BANK_SIZE * (pin->pin >> 5));
  uint32_t mask = 1 << (pin->pin & 0x1f);

  if(value)
    *(volatile uint32_t *)(regs + GPIO_REG_SOR) = mask;
  else
    *(volatile uint32_t *)(regs + GPIO_REG_COR) = mask;
}

static int wb_gpio_in(const struct gpio_pin *pin)
{
  void *base = (void*)pin->rwops->priv;
  volatile uint32_t *psr = base + (GPIO_BANK_SIZE * (pin->pin >> 5)) + GPIO_REG_PSR;
  uint32_t mask = 1 << (pin->pin & 0x1f);

  return (*psr & mask) ? 1 : 0;
}


struct gpio_rwops gpio_main = {
    BASE_GPIO,
    wb_gpio_set_dir,
    wb_gpio_out,
    wb_gpio_in
};
