#ifndef __GPIO_H
#define __GPIO_H

#include "board.h"

struct gpio_pin;

typedef void (*set_dir_func)(const struct gpio_pin *, int);
typedef void (*set_out_func)(const struct gpio_pin *, int);
typedef int (*read_pin_func)(const struct gpio_pin *);

struct gpio_rwops
{
    void* priv;
    set_dir_func set_dir;
    set_out_func set_out;
    read_pin_func read_pin;
};

struct gpio_pin
{
    const struct gpio_rwops *rwops;
    int pin;
};

extern struct gpio_rwops gpio_main;
extern struct gpio_rwops gpio_rtm;

static inline void gpio_set_dir(const struct gpio_pin *pin, int dir)
{
    pin->rwops->set_dir(pin, dir);
}

static inline void gpio_out(const struct gpio_pin *pin, int value)
{
    pin->rwops->set_out(pin, value);
}

static inline int gpio_in(const struct gpio_pin *pin)
{
    return pin->rwops->read_pin(pin);
}

static const struct gpio_pin gpio_pin_clk_muxab_sel0 = { &gpio_main, 23 };
static const struct gpio_pin gpio_pin_clk_muxab_sel1 = { &gpio_main, 24 };

static const struct gpio_pin gpio_pin_clk_mux1a_sel0 = { &gpio_main, 25 };
static const struct gpio_pin gpio_pin_clk_mux1a_sel1 = { &gpio_main, 26 };

static const struct gpio_pin gpio_pin_clk_mux1b_sel0 = { &gpio_main, 27 };
static const struct gpio_pin gpio_pin_clk_mux1b_sel1 = { &gpio_main, 28 };

static const struct gpio_pin gpio_pin_clk_mux2a_sel0 = { &gpio_main, 29 };
static const struct gpio_pin gpio_pin_clk_mux2a_sel1 = { &gpio_main, 30 };

static const struct gpio_pin gpio_pin_clk_mux2b_sel0 = { &gpio_main, 32 };
static const struct gpio_pin gpio_pin_clk_mux2b_sel1 = { &gpio_main, 33 };

static const struct gpio_pin gpio_pin_clk_mux_dac_sel0 = { &gpio_main, 57 };
static const struct gpio_pin gpio_pin_clk_mux_dac_sel1 = { &gpio_main, 58 };

static const struct gpio_pin gpio_pin_si_rst_n = { &gpio_main, 16 };
static const struct gpio_pin gpio_pin_si_spi_sclk = { &gpio_main, 20 };
static const struct gpio_pin gpio_pin_si_spi_di = { &gpio_main, 21 };
static const struct gpio_pin gpio_pin_si_spi_do = { &gpio_main, 22 }; // Si3
static const struct gpio_pin gpio_pin_si_spi_cs_n1 = { &gpio_main, 19 };


static const struct gpio_pin gpio_pin_div_spi_sclk = { &gpio_main, 34 };
static const struct gpio_pin gpio_pin_div_spi_dio = { &gpio_main, 35 };
static const struct gpio_pin gpio_pin_div_spi_cs_n1 = { &gpio_main, 36 };
static const struct gpio_pin gpio_pin_div_spi_cs_n2 = { &gpio_main, 37 };
static const struct gpio_pin gpio_pin_div_function = { &gpio_main, 40 };

static const struct gpio_pin gpio_pin_adc_sync = { &gpio_main, 41};
static const struct gpio_pin gpio_pin_adc_oeb_l = { &gpio_main, 42};
static const struct gpio_pin gpio_pin_adc_pwdn = { &gpio_main, 43};
static const struct gpio_pin gpio_pin_adc_spi_cs_l1 = { &gpio_main, 45 };
static const struct gpio_pin gpio_pin_adc_spi_cs_l2 = { &gpio_main, 46 };
static const struct gpio_pin gpio_pin_adc_spi_cs_l3 = { &gpio_main, 47 };
static const struct gpio_pin gpio_pin_adc_spi_cs_l4 = { &gpio_main, 48 };
static const struct gpio_pin gpio_pin_adc_spi_cs_l5 = { &gpio_main, 49 };
static const struct gpio_pin gpio_pin_adc_spi_sclk = { &gpio_main, 50 };
static const struct gpio_pin gpio_pin_adc_spi_dio = { &gpio_main, 51 };


static const struct gpio_pin gpio_pin_rtm_usr_scl = { &gpio_main, 55 };
static const struct gpio_pin gpio_pin_rtm_usr_sda = { &gpio_main, 56 };

static const struct gpio_pin gpio_pin_leds[] =
{
{  &gpio_main, 0 },
{  &gpio_main, 1 },
{  &gpio_main, 2 },
{  &gpio_main, 3 },
{  &gpio_main, 4 },
{  &gpio_main, 5 },
{  &gpio_main, 6 },
{  &gpio_main, 7 },
{  &gpio_main, 8 },
{  &gpio_main, 9 },
{  &gpio_main, 10 },
{  &gpio_main, 11 },
{  &gpio_main, 12 },
{  &gpio_main, 13 },
{  &gpio_main, 14 },
{  &gpio_main, 15 }
};

#endif
